#ifdef COMPUTE_SHADER

#define KERNEL_RADIUS 4
#define SHARED_TEXELS 40
#define PI 3.1415926538

// 32 + kernelradius * 2
layout(local_size_y = 40) in;

layout(rgba16f, binding = 0) uniform image2D image;
layout(rgba16f, binding = 1) uniform image2D linear_depth;

//uniform float kernel[5] = float[] (0.2270270270, 0.1945945946, 0.1216216216, 0.0540540541, 0.0162162162);
uniform float kernel[5] = float[] (0.20236, 0.179044, 0.124009, 0.067234, 0.028532);
uniform ivec2 img_size;

shared vec4 shared_texels[SHARED_TEXELS];
shared float shared_depth_texels[SHARED_TEXELS];

float gauss(float dist, float sd)	{
	float tmp = 2 * sd * sd;
	return 1 / (sqrt(tmp * PI)) * exp(-(dist * dist) / tmp);
}

void main()
{
	ivec2 id = ivec2(gl_GlobalInvocationID.xy);
	ivec2 lookup = clamp(id - ivec2(0, KERNEL_RADIUS * (gl_WorkGroupID.y * 2 + 1)), ivec2(0,0), img_size - 1);
	
	shared_texels[gl_LocalInvocationID.y] = imageLoad(image, lookup);
	shared_depth_texels[gl_LocalInvocationID.y] = imageLoad(linear_depth, lookup).x;
	memoryBarrierShared();
	barrier();
	
	if(KERNEL_RADIUS <= gl_LocalInvocationID.y && gl_LocalInvocationID.y < 40 - KERNEL_RADIUS){
		vec3 result = vec3(0.0);
		float w_p = 0.0;
		float sd = 1.0;
		//vec3 gray_interp = vec3(0.2126, 0.7152, 0.0722);
		vec3 current_texel = shared_texels[gl_LocalInvocationID.y].rgb;
		float current_depth = shared_depth_texels[gl_LocalInvocationID.y];
		//float current_gray_val = dot(current_texel, gray_interp);
		float current_weight = kernel[0] * gauss(0.0, sd); 
		result += current_texel * current_weight;
		w_p += current_weight;

		for(int i = 1; i <= KERNEL_RADIUS; i++){
			vec3 texel = shared_texels[gl_LocalInvocationID.y - i].rgb;
			//current_weight = kernel[i] * gauss(length(current_texel - texel), sd);
			//current_weight = kernel[i] * gauss(current_gray_val - dot(texel, gray_interp), sd);
			current_weight = kernel[i] * gauss(current_depth - shared_depth_texels[gl_LocalInvocationID.y - i], sd);
			result += texel * current_weight;
			w_p += current_weight;

			texel = shared_texels[gl_LocalInvocationID.y + i].rgb;
			//current_weight = kernel[i] * gauss(length(current_texel - texel), sd);
			//current_weight = kernel[i] * gauss(current_gray_val - dot(texel, gray_interp), sd);
			current_weight = kernel[i] * gauss(current_depth - shared_depth_texels[gl_LocalInvocationID.y + i], sd);
			result += texel * current_weight;
			w_p += current_weight;
		}
		imageStore(image, lookup, vec4(result/w_p, 1.0));
	}
	
}

#endif
