#ifdef VERTEX_SHADER

layout (location = 0) in vec3 m_vertex;

layout(std430, binding=5) buffer Model_in {
	mat4 model_matrix[];
};

void main()
{
    gl_Position = model_matrix[gl_InstanceID] * vec4(m_vertex, 1.0);
}

#endif

#ifdef GEOMETRY_SHADER

layout (triangles) in;
layout (triangle_strip, max_vertices = 18) out;

layout (std140, binding = 4) uniform View_proj_matrix {
    mat4 view_proj[6];
};

out vec3 w_position;

void main(){
    // face
    for (int f = 0; f < 6; f++){
        gl_Layer = f;
        for (int i = 0; i < 3; i++) {
           w_position = gl_in[i].gl_Position.xyz;
           gl_Position = view_proj[f] * vec4(w_position, 1.0);
           EmitVertex();
        }
        EndPrimitive();
    }
}

#endif

#ifdef FRAGMENT_SHADER

#define NUM_OF_POINTLIGHTS 3

in vec3 w_position;

struct PointLight{
    vec3 position;
    vec3 color;
    vec3 attenuation;
    vec2 near_far_plane;
    vec2 padding;
    vec2 tau_phi;
};

layout (std140, binding = 3) uniform PointLights {
    PointLight pointLights[NUM_OF_POINTLIGHTS];
};

uniform uint light_index;
uniform vec3 w_light_position;

void main()
{
    float dist = length(w_position - w_light_position);
    gl_FragDepth = dist / pointLights[light_index].near_far_plane.y;
}

#endif
