#ifdef VERTEX_SHADER

void main()
{
}

#endif

#ifdef GEOMETRY_SHADER

layout (points) in;
layout (triangle_strip, max_vertices = 3) out;
out vec2 f_uv;

void main(){
    gl_Position = vec4(-1.0, -1.0, 0.0, 1.0);
    f_uv = vec2(0.0, 0.0);
    EmitVertex();
    gl_Position = vec4(3.0, -1.0, 0.0, 1.0);
    f_uv = vec2(2.0, 0.0);
    EmitVertex();
    gl_Position = vec4(-1.0, 3.0, 0.0, 1.0);
    f_uv = vec2(0.0, 2.0);
    EmitVertex();
    EndPrimitive();
}

#endif

#ifdef FRAGMENT_SHADER

in vec2 f_uv;

//uniform vec3 color;
layout (std140, binding = 1) uniform Camera {
    mat4 view;
    mat4 view_inv;
    mat4 projection;
    mat4 projection_inv;
    mat4 viewProjection;
    vec3 camera_position;
    vec4 fov_near_far;
    uvec4 resolution;
    float time;
    vec2 padding;
    float relief_factor;
};

uniform sampler2D texture_normals_and_mask;
uniform sampler2D texture_depth;

out vec4 FragColor;

float linearize_depth(float d,float zNear,float zFar)
{
    return zNear * zFar / (zFar + d * (zNear - zFar));
}

// from https://stackoverflow.com/questions/32227283/getting-world-position-from-depth-buffer-value
vec3 WorldPosFromDepth(float depth, vec2 uv) {
    float z = depth * 2.0 - 1.0;

    vec4 clipSpacePosition = vec4(uv * 2.0 - 1.0, z, 1.0);
    vec4 viewSpacePosition = projection_inv * clipSpacePosition;

    // Perspective division
    viewSpacePosition /= viewSpacePosition.w;

    vec4 worldSpacePosition = view_inv * viewSpacePosition;

    return worldSpacePosition.xyz;
}

void main()
{
    FragColor = vec4(1.0f);

    vec4 normals_and_mask = texture(texture_normals_and_mask, f_uv);
    float mask = normals_and_mask.a;
    vec3 normal = normals_and_mask.xyz;
    float depth_non_linear = texture(texture_depth, f_uv).r;
    float depth_linear = linearize_depth(depth_non_linear, fov_near_far.y, fov_near_far.z);

    if (mask < 0.5) {
        FragColor = vec4(0.0);
        //return;
    }

    float max_distance = 15.0;
    float step_resolution = 0.3; // Between 0 - 1
    int steps = 10;
    float thickness = 0.5;

    vec2 texture_size = textureSize(texture_depth, f_uv);

    vec3 pos_world = WorldPosFromDepth(depth_non_linear, f_uv);
    vec3 view_dir = normalize(pos_world - camera_position);
    vec3 reflection_dir = normalize(reflect(view_direction, normal));

    vec4 start_view = view * vec4(pos_world + reflection * 0), 1);
    vec4 end_view = view * vec4(pos_world + reflection * max_distance), 1);

    vec4 start_frag = start_view;
    start_frag = projection * start_frag;
    start_frag.xyz = start_frag.w;
    start_frag.xy = start_frag.xy * 0.5 + 0.5;
    start_frag.xy *= texture_size;

    vec4 end_frag = end_view;
    end_frag = projection * end_frag;
    end_frag.xyz = end_frag.w;
    end_frag.xy = end_frag.xy * 0.5 + 0.5;
    end_frag.xy *= texture_size;

    vec2 frag = start_frag.xy;
    vec2 uv_current = frag / texture_size;

    vec2 frag_delta = end_frag - start_frag;
    float use_x = abs(frag_delta.x) > abs(frag_delta.y) ? 1.0 : 0.0;
    float delta = mix(abs(frag_delta.y), abs(frag_delta.x), use_x) * clamp(step_resolution, 0, 1);

    vec2 increment = frag_delta / max(delta, 0.001);
    
    float current_depth = depth_non_linear;

    for (int i = 0; i < int(delta); i++) {
        frag += increment;
        uv_current = frag / texture_size;
        float scene_depth = texture(texture_depth, uv_current);
    }

    // Visualization
    float val = screen_dir.y;
    val = reflection_screen.x;
    if (val > 0.0) FragColor.rgb = vec3(0.0, val, 0.0);
    else FragColor.rgb = vec3(-val, 0.0, 0.0);

    // SCREEN SPACE REFLECTIONS
    /*
        1. Calculate 2D direction of reflection
            - Get 3D position of fragment
            - Get direction vector
            - Reflect along normal
            - Project Reflection to view space
        2. March along 2D ray, compare ray depth with scene depth, check if there is a collision
        3. Use color of the intersected position as reflection color
        
        Check if we are still in bounds/Not going out of the screen
        Fallback color/cubemap/something required
    */

    FragColor += vec4(depth_linear)*0.0001; // Delete later 
}

#endif


/*

    FragColor.rgba = normals_and_mask;
    //FragColor.rgba = vec4(vec3(pow(linear_depth/40, 5)), 1.0);

    vec3 pos_world = WorldPosFromDepth(depth_non_linear, f_uv);
    FragColor.rgb = vec3(pos_world.z);
    vec3 view_dir_world = normalize(pos_world - camera_position);

    vec3 reflection_world = reflect(view_dir_world, normal);

    vec3 pos_start_world = pos_world;
    vec3 pos_end_world = pos_world + reflection_world;

    vec4 pos_start_clip = viewProjection * vec4(pos_start_world, 1.0);
    vec4 pos_end_clip = viewProjection * vec4(pos_end_world, 1.0);

    vec2 pos_start_screen = (pos_start_clip.xy / pos_start_clip.w).xy;
    vec2 pos_end_screen = (pos_end_clip.xy / pos_end_clip.w).xy;

    vec2 screen_dir = normalize(pos_end_screen - pos_start_screen);

    // TEST
    vec3 reflection_view = vec3(view * vec4(reflection_world, 0.0));
    vec4 reflection_projected_homogen = projection * vec4(reflection_world, 0.0);
    vec3 reflection_projected = reflection_projected_homogen.xyz / reflection_projected_homogen.w;
    //vec3 reflection_projected = reflection_projected_homogen.xyz;
    vec2 reflection_screen = normalize(reflection_view.xy);
    //reflection_screen = normalize(reflection_projected_homogen.xy);
    // TEST END



*/



