#ifdef VERTEX_SHADER

void main()
{
}

#endif

#ifdef GEOMETRY_SHADER

layout (points) in;
layout (triangle_strip, max_vertices = 3) out;
out vec2 uv;

void main(){
    gl_Position = vec4(-1.0, -1.0, 0.0, 1.0);
    uv = vec2(0.0, 0.0);
    EmitVertex();
    gl_Position = vec4(3.0, -1.0, 0.0, 1.0);
    uv = vec2(2.0, 0.0);
    EmitVertex();
    gl_Position = vec4(-1.0, 3.0, 0.0, 1.0);
    uv = vec2(0.0, 2.0);
    EmitVertex();
    EndPrimitive();
}

#endif

#ifdef FRAGMENT_SHADER

#define NUM_OF_POINTLIGHTS 2

in vec2 uv;

//uniform vec3 color;
layout (std140, binding = 1) uniform Camera {
    mat4 view;
    mat4 view_inv;
    mat4 projection;
    mat4 projection_inv;
    mat4 viewProjection;
    vec3 camera_position;
    vec4 fov_near_far;
    uvec4 resolution;
};

struct PointLight{
    vec3 position;
    vec3 color;
    vec3 attenuation;
};

layout (std140, binding = 3) uniform PointLights {
    PointLight pointLights[NUM_OF_POINTLIGHTS];
};

uniform sampler2D depth_texture;
uniform samplerCube shadow_map[NUM_OF_POINTLIGHTS];

out vec4 FragColor;

float linearize_depth(float d,float zNear,float zFar)
{
    return zNear * zFar / (zFar + d * (zNear - zFar));
}

// from https://stackoverflow.com/questions/32227283/getting-world-position-from-depth-buffer-value
vec3 WorldPosFromDepth(float depth, vec2 uv) {
    float z = depth * 2.0 - 1.0;

    vec4 clipSpacePosition = vec4(uv * 2.0 - 1.0, z, 1.0);
    vec4 viewSpacePosition = projection_inv * clipSpacePosition;

    // Perspective division
    viewSpacePosition /= viewSpacePosition.w;

    vec4 worldSpacePosition = view_inv * viewSpacePosition;

    return worldSpacePosition.xyz;
}

// shadow map function
float is_lit(vec3 w_position, int i) {
    // TODO dynamic far_plane
    float far_plane = 100.0;
    vec3 light_to_pos = w_position - pointLights[i].position;
    float depth_dist = texture(shadow_map[i], light_to_pos).r * far_plane;
    float bias = 0.05;
    return length(light_to_pos) - bias > depth_dist? 0.0 : 1.0;
}


void main()
{
    //float stepsize = 0.1;
    float NUM_OF_SAMPLES = 100;
    FragColor = vec4(vec3(0.0),1.0);
    //float col = linearize_depth(texture(depth, uv).r, fov_near_far.y, 0.8);
    vec3 w_frag_position = WorldPosFromDepth(texture(depth_texture, uv).r, uv);
    vec3 w_camera_to_frag = w_frag_position - camera_position;
    float raymarch_distance_worldspace =  length(w_camera_to_frag);
    //float n = raymarch_distance_worldspace / stepsize;
    float stepsize = raymarch_distance_worldspace / NUM_OF_SAMPLES;
    vec3 delta_worldspace = w_camera_to_frag / NUM_OF_SAMPLES;

    float tau = 0.1;
    float PI_RCP = 1 / 3.141592;
    float phi = 20.0;

    for(int i = 0; i < NUM_OF_POINTLIGHTS; i++){
        vec3 sample_pos = w_frag_position;
        float cont = 0.0;
        for (float l = raymarch_distance_worldspace; l > stepsize; l -= stepsize){
            float lit = is_lit(sample_pos, i);
            float d = length(sample_pos - pointLights[i].position);
            float att = 1.0 / (pointLights[i].attenuation.x + d * pointLights[i].attenuation.y + d * d * pointLights[i].attenuation.z);
            float d_rcp = 1.0 / d;
            cont += att * tau * (lit * (phi * 0.25 * PI_RCP) * d_rcp * d_rcp) * exp(-d*tau) * exp(-l*tau) * stepsize;
            sample_pos -= delta_worldspace;
        }
        FragColor += vec4(vec3(pointLights[i].color * cont), 0.0);
    }
}

#endif

